@@ -16,14 +16,16 @@ import inspect
 import re
 
 import netaddr
+from os_ken.lib.packet import ether_types
+from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import netutils
 from oslo_utils import strutils
 from oslo_utils import uuidutils
-import six
 from webob import exc
 
 from neutron_lib._i18n import _
+from neutron_lib.api import converters
 from neutron_lib import constants
 from neutron_lib import exceptions as n_exc
 from neutron_lib.plugins import directory
@@ -161,14 +163,14 @@ def validate_values(data, valid_values=None, valid_values_display=None):
                        {'data': data, 'valid_values': valid_values_display})
                 LOG.debug(msg)
                 return msg
-        except TypeError:
+        except TypeError as e:
             # This is a programming error
             msg = (_("'data' of type '%(typedata)s' and 'valid_values' "
                      "of type '%(typevalues)s' are not "
                      "compatible for comparison") %
                    {'typedata': type(data),
                     'typevalues': type(valid_values)})
-            raise TypeError(msg)
+            raise TypeError(msg) from e
     else:
         # This is a programming error
         msg = (_("'valid_values' does not support membership operations"))
@@ -226,7 +228,7 @@ def validate_string(data, max_len=None):
         the given max_len. Otherwise a human readable message indicating why
         the data is invalid.
     """
-    if not isinstance(data, six.string_types):
+    if not isinstance(data, str):
         msg = _("'%s' is not a valid string") % data
         LOG.debug(msg)
         return msg
@@ -376,7 +378,7 @@ def validate_mac_address(data, valid_values=None):
         valid_mac = False
 
     if valid_mac:
-        valid_mac = (not netaddr.EUI(data) in
+        valid_mac = (netaddr.EUI(data) not in
                      map(netaddr.EUI, constants.INVALID_MAC_ADDRESSES))
     # TODO(arosen): The code in this file should be refactored
     # so it catches the correct exceptions. validate_no_whitespace
@@ -585,7 +587,7 @@ def validate_hostroutes(data, valid_values=None):
     :param data: The data to validate. To be valid it must be a list like
         structure of host route dicts, each containing 'destination' and
         'nexthop' key values.
-    :param valid_values: Not used!
+    :param valid_values: A list of valid keys or None
     :returns: None if data is a valid list of unique host route dicts,
         otherwise a human readable message indicating why validation failed.
     """
@@ -597,6 +599,8 @@ def validate_hostroutes(data, valid_values=None):
     expected_keys = ['destination', 'nexthop']
     hostroutes = []
     for hostroute in data:
+        if hostroute and 'bfd_monitor_id' in hostroute:
+            expected_keys.append('bfd_monitor_id')
         msg = _verify_dict_keys(expected_keys, hostroute)
         if msg:
             return msg
@@ -610,6 +614,10 @@ def validate_hostroutes(data, valid_values=None):
             msg = _("Duplicate hostroute '%s'") % hostroute
             LOG.debug(msg)
             return msg
+        if 'bfd_monitor_id' in hostroute:
+            msg = validate_uuid(hostroute['bfd_monitor_id'])
+            if msg:
+                return msg
         hostroutes.append(hostroute)
 
 
@@ -867,8 +875,8 @@ def _extract_validator(key_validator):
                 return (validator_name,
                         validators[validator_name],
                         validator_params)
-            except KeyError:
-                raise UndefinedValidator(validator_name)
+            except KeyError as e:
+                raise UndefinedValidator(validator_name) from e
     return None, None, None
 
 
@@ -1115,18 +1123,94 @@ def validate_subnet_service_types(service_types, valid_values=None):
         prefixes += constants.DEVICE_OWNER_COMPUTE_PREFIX
 
         for service_type in service_types:
-            if not isinstance(service_type, six.text_type):
+            if not isinstance(service_type, str):
                 raise n_exc.InvalidInputSubnetServiceType(
                     service_type=service_type)
-            elif not service_type.startswith(tuple(prefixes)):
+            if not service_type.startswith(tuple(prefixes)):
                 raise n_exc.InvalidSubnetServiceType(service_type=service_type)
 
 
+def validate_ethertype(ethertype, valid_values=None):
+    """Validates ethertype is a valid ethertype.
+
+    :param ethertype: Ethertype to validate.
+    :returns: None if data is a valid ethertype.  Otherwise a human-readable
+        message indicating that the data is not a valid ethertype.
+    """
+
+    if cfg.CONF.sg_filter_ethertypes:
+        os_ken_ethertypes = ether_types.__dict__
+        try:
+            # Use the ethertype constants from os_ken with the special
+            # exception of 'IPv4', which appears as 'ETH_TYPE_IP' in os_ken.
+            # Be case-insensitive.
+            ethertype_str = str(ethertype).upper()
+            if ethertype_str == "IPV4":
+                ethertype_str = "IP"
+            ethertype_name = 'ETH_TYPE_' + ethertype_str
+            ethertype = os_ken_ethertypes.get(ethertype_name, ethertype)
+        except TypeError:
+            # Value of ethertype cannot be coerced into a string, like None
+            pass
+        try:
+            if isinstance(ethertype, str):
+                ethertype = int(ethertype, 16)
+            if (isinstance(ethertype, int) and
+                    constants.ETHERTYPE_MIN <= ethertype <=
+                    constants.ETHERTYPE_MAX):
+                return None
+        except ValueError:
+            pass
+        msg = _("Ethertype %s is not a two octet hexadecimal "
+                "number or ethertype name.") % ethertype
+        LOG.debug(msg)
+        return msg
+    else:
+        if ethertype in constants.VALID_ETHERTYPES:
+            return None
+        valids = ','.join(map(str, constants.VALID_ETHERTYPES))
+        msg = _("Ethertype %(ethertype)s is not a valid ethertype, must be "
+                "one of %(valid_ethertypes)s.") % {'ethertype': ethertype,
+                                                   'valid_ethertypes': valids}
+        LOG.debug(msg)
+        return msg
+
+
+def validate_external_gw_info(data, valid_values=None):
+    """Validate data is an external_gateway_info.
+
+    :param data: The data to validate.
+    :param valid_values: Not used!
+    :returns: None if valid, error string otherwise.
+    """
+    return validate_dict_or_nodata(
+        data,
+        key_specs={
+            'network_id': {'type:uuid': None, 'required': True},
+            'external_fixed_ips': {'type:fixed_ips': None, 'required': False},
+            'enable_snat': {'type:boolean': None, 'required': False,
+                            'convert_to': converters.convert_to_boolean},
+        }
+    )
+
+
+def validate_external_gw_info_list(data, valid_values=None):
+    """Validate data is a list of external_gateway_info.
+
+    :param data: The data to validate.
+    :param valid_values: Not used!
+    :returns: None if valid, error string otherwise.
+    """
+    if data is not None:
+        return _validate_list_of_items(validate_external_gw_info, data)
+
+
 # Dictionary that maintains a list of validation functions
 validators = {'type:dict': validate_dict,
               'type:dict_or_none': validate_dict_or_none,
               'type:dict_or_empty': validate_dict_or_empty,
               'type:dict_or_nodata': validate_dict_or_nodata,
+              'type:ethertype': validate_ethertype,
               'type:fixed_ips': validate_fixed_ips,
               'type:hostroutes': validate_hostroutes,
               'type:ip_address': validate_ip_address,
@@ -1167,7 +1251,9 @@ validators = {'type:dict': validate_dict,
               'type:service_plugin_type': validate_service_plugin_type,
               'type:list_of_subnets_or_none': validate_subnet_list_or_none,
               'type:list_of_subnet_service_types':
-                  validate_subnet_service_types
+                  validate_subnet_service_types,
+              'type:external_gw_info': validate_external_gw_info,
+              'type:external_gw_info_list': validate_external_gw_info_list,
               }
 
 
